@@ -2196,7 +2196,9 @@ whether it is a file(/result) or a directory (/result/)."
 				"/usr/local/lib/X11R5/X11/"
 				"/usr/X11/lib/X11/"
 				"/usr/lib/X11/"
+				"/usr/share/X11/"
 				"/usr/local/lib/X11/"
+				"/usr/local/share/X11/"
 				"/usr/X386/lib/X11/"
 				"/usr/x386/lib/X11/"
 				"/usr/XFree86/lib/X11/"
