@@ -51,7 +51,7 @@ Boston, MA 02111-1307, USA.  */
 #include <dmalloc.h>
 #endif
 
-/* simple, naieve integer maximum */
+/* simple, naive integer maximum */
 #ifndef max
 #define max(a,b) ((a)>(b)?(a):(b))
 #endif
@@ -83,25 +83,25 @@ xlwMenuResources[] =
      /* We must use an iso8859-1 font here, or people without $LANG set lose.
 	It's fair to assume that those who do have $LANG set also have the
 	*fontList resource set, or at least know how to deal with this. */
-     XtRString, "-*-helvetica-bold-r-*-*-*-120-*-*-*-*-iso8859-1"},
+     XtRString, (XtPointer) "-*-helvetica-bold-r-*-*-*-120-*-*-*-*-iso8859-1"},
 #else
   {XtNfont,  XtCFont, XtRFontStruct, sizeof(XFontStruct *),
-     offset(menu.font), XtRString, "XtDefaultFont"},
+     offset(menu.font), XtRString, (XtPointer) "XtDefaultFont"},
 # ifdef USE_XFONTSET
   /* #### Consider using the same method as for Motif; see the comment in
      XlwMenuInitialize(). */
   {XtNfontSet,  XtCFontSet, XtRFontSet, sizeof(XFontSet),
-     offset(menu.font_set), XtRString, "XtDefaultFontSet"},
+     offset(menu.font_set), XtRString, (XtPointer) "XtDefaultFontSet"},
 # endif
 #endif
   {XtNforeground, XtCForeground, XtRPixel, sizeof(Pixel),
-     offset(menu.foreground), XtRString, "XtDefaultForeground"},
+     offset(menu.foreground), XtRString, (XtPointer) "XtDefaultForeground"},
   {XtNbuttonForeground, XtCButtonForeground, XtRPixel, sizeof(Pixel),
-     offset(menu.button_foreground), XtRString, "XtDefaultForeground"},
+     offset(menu.button_foreground), XtRString, (XtPointer) "XtDefaultForeground"},
   {XtNhighlightForeground, XtCHighlightForeground, XtRPixel, sizeof(Pixel),
-     offset(menu.highlight_foreground), XtRString, "XtDefaultForeground"},
+     offset(menu.highlight_foreground), XtRString, (XtPointer) "XtDefaultForeground"},
   {XtNtitleForeground, XtCTitleForeground, XtRPixel, sizeof(Pixel),
-     offset(menu.title_foreground), XtRString, "XtDefaultForeground"},
+     offset(menu.title_foreground), XtRString, (XtPointer) "XtDefaultForeground"},
   {XtNmargin, XtCMargin, XtRDimension,  sizeof(Dimension),
      offset(menu.margin), XtRImmediate, (XtPointer)2},
   {XmNmarginWidth, XmCMarginWidth, XmRHorizontalDimension, sizeof(Dimension),
@@ -197,7 +197,7 @@ XlwMenuClassRec xlwMenuClassRec =
     XtNumber(xlwMenuResources),		/* resource_count	  */
     NULLQUARK,				/* xrm_class		  */
     TRUE,				/* compress_motion	  */
-    TRUE,				/* compress_exposure	  */
+    XtExposeCompressMaximal,		/* compress_exposure	  */
     TRUE,				/* compress_enterleave    */
     FALSE,				/* visible_interest	  */
     XlwMenuDestroy,			/* destroy		  */
@@ -348,7 +348,8 @@ close_to_reference_time (Widget w, Time reference_time, XEvent *ev)
 {
   return
     reference_time &&
-    (ev->xbutton.time - reference_time < XtGetMultiClickTime (XtDisplay (w)));
+    (ev->xbutton.time - reference_time
+     < (Time) XtGetMultiClickTime (XtDisplay (w)));
 }
 
 /* Size code */
@@ -459,7 +460,7 @@ string_width_u (XlwMenuWidget mw,
 }
 
 static void
-massage_resource_name (CONST char *in, char *out)
+massage_resource_name (const char *in, char *out)
 {
   /* Turn a random string into something suitable for using as a resource.
      For example:
@@ -481,16 +482,26 @@ massage_resource_name (CONST char *in, char *out)
   Boolean firstp = True;
   while (*in)
     {
-      char ch = massaged_resource_char[(unsigned char) *in++];
-      if (ch)
+      if (*in == '%' && *(in + 1) == '_')
+	in += 2;
+      else
 	{
-	  int int_ch = (int) (unsigned char) ch;
-	  *out++ = firstp ? tolower (int_ch) : toupper (int_ch);
-	  firstp = False;
-	  while ((ch = massaged_resource_char[(unsigned char) *in++]) != '\0')
-	    *out++ = ch;
-	  if (!*(in-1))		/* Overshot the NULL byte? */
-	    break;
+	  char ch;
+
+	  if (*in == '%' && *(in + 1) == '%')
+	    in++;
+	  ch = massaged_resource_char[(unsigned char) *in++];
+	  if (ch)
+	    {
+	      int int_ch = (int) (unsigned char) ch;
+	      *out++ = firstp ? tolower (int_ch) : toupper (int_ch);
+	      firstp = False;
+	      while ((ch = massaged_resource_char[(unsigned char) *in++])
+		     != '\0')
+		*out++ = ch;
+	      if (!*(in-1))		/* Overshot the NULL byte? */
+		break;
+	    }
 	}
     }
   *out = 0;
@@ -508,23 +519,21 @@ nameResource[] =
     0, XtRImmediate, 0 }
 };
 
-/*
- *    This function looks through string searching for parameter
- *    inserts of the form:
- *    %[padding]1
- *    padding is space (' ') or dash ('-') characters meaning
- *    padding to the left or right of the inserted parameter.
- *    In essence all %1 strings are replaced by value in the return
- *    value (which the caller is expected to free).
- *    %% means insert one % (like printf).
- *    %1 means insert value.
- *    %-1 means insert value followed by one space. The latter is
- *    not inserted if value is a zero length string.
- */
+/* This function searches STRING for parameter inserts of the form:
+       %[padding]1
+   padding is either space (' ') or dash ('-') meaning
+   padding to the left or right of the inserted parameter.
+   In essence, all %1 strings are replaced by VALUE in the return value.
+   The caller is expected to free the return value using XtFree().
+   %% means insert one % (like printf).
+   %1 means insert VALUE.
+   %-1 means insert VALUE followed by one space. The latter is
+   not inserted if VALUE is a zero length string.
+*/
 static char*
-parameterize_string (CONST char *string, CONST char *value)
+parameterize_string (const char *string, const char *value)
 {
-  char *percent;
+  const char *percent;
   char *result;
   unsigned int done = 0;
   unsigned int ntimes;
@@ -533,24 +542,25 @@ parameterize_string (CONST char *string, CONST char *value)
     {
       result = XtMalloc(1);
       result[0] = '\0';
-      return (result);
+      return result;
     }
 
   if (!value)
     value = "";
 
-  for (ntimes = 1, result = (char *) string; (percent = strchr(result, '%'));
+  for (ntimes = 1, percent = string;
+       (percent = strchr (percent, '%'));
        ntimes++)
-    result = &percent[1];
+    percent++;
 
   result = XtMalloc ((ntimes * strlen(value)) + strlen(string) + 4);
   result[0] = '\0';
 
-  while ((percent = strchr(string, '%')))
+  while ((percent = strchr (string, '%')))
     {
       unsigned int left_pad;
       unsigned int right_pad;
-      char *p;
+      const char *p;
 
       if (percent[1] == '%')
 	{	/* it's a real % */
@@ -681,7 +691,7 @@ resource_widget_value (XlwMenuWidget mw, widget_value *val)
 
 /* Unused */
 #if 0
-/* These two routines should be a seperate file..djw */
+/* These two routines should be a separate file..djw */
 static char *
 xlw_create_localized_string (Widget w,
 			     char *name,
@@ -869,8 +879,8 @@ string_draw_u (XlwMenuWidget mw,
 #endif
 )
 {
-int i,s=0;
-char *chars;
+  int i, s = 0;
+  char *chars;
 
 #ifdef NEED_MOTIF
   chars = "";
@@ -879,8 +889,8 @@ char *chars;
 #else
   chars = string;
 #endif
-  for (i=0;chars[i];++i) {
-      if (chars[i]=='%'&&chars[i+1]=='_') {
+  for (i=0; chars[i]; ++i) {
+      if (chars[i] == '%' && chars[i+1] == '_') {
 	  int w;
 
 	  x += string_draw_range (mw, window, x, y, gc, chars, s, i);
@@ -1479,7 +1489,7 @@ print_widget_value (widget_value *wv, int just_one, int depth)
       print_widget_value (wv->next, 0, depth);
     }
 }
-#endif
+#endif /* SLOPPY_TYPES < 2 */
 
 static Boolean
 all_dashes_p (char *s)
@@ -1493,30 +1503,29 @@ all_dashes_p (char *s)
     return True;
   return False;
 }
-#endif
+#endif /* SLOPPY_TYPES */
 
 static widget_value_type
 menu_item_type (widget_value *val)
 {
   if (val->type != UNSPECIFIED_TYPE)
     return val->type;
-  else
-    {
 #if SLOPPY_TYPES
-      if (all_dashes_p (val->name))
-	return SEPARATOR_TYPE;
-      else if (val->name && val->name[0] == '\0') /* push right */
-	return PUSHRIGHT_TYPE;
-      else if (val->contents) /* cascade */
-	return CASCADE_TYPE;
-      else if (val->call_data) /* push button */
-	return BUTTON_TYPE;
-      else
-	return TEXT_TYPE;
+  else if (all_dashes_p (val->name))
+    return SEPARATOR_TYPE;
+  else if (val->name && val->name[0] == '\0') /* push right */
+    return PUSHRIGHT_TYPE;
+  else if (val->contents) /* cascade */
+    return CASCADE_TYPE;
+  else if (val->call_data) /* push button */
+    return BUTTON_TYPE;
+  else
+    return TEXT_TYPE;
 #else
+  else
     abort();
+  return UNSPECIFIED_TYPE; /* Not reached */
 #endif
-    }
 }
 
 static void
@@ -1532,7 +1541,7 @@ label_button_size (XlwMenuWidget mw,
 	     2 * mw->menu.vertical_margin +
 	     2 * mw->menu.shadow_thickness);
   /* no left column decoration */
-  *toggle_width = mw->menu.horizontal_margin + mw->menu.shadow_thickness;;
+  *toggle_width = mw->menu.horizontal_margin + mw->menu.shadow_thickness;
 
   *label_width  = string_width_u (mw, resource_widget_value (mw, val));
   *bindings_width =  mw->menu.horizontal_margin + mw->menu.shadow_thickness;
@@ -1876,7 +1885,7 @@ radio_button_draw (XlwMenuWidget mw,
 
 static struct _shadow_names
 {
-  CONST char *      name;
+  const char *      name;
   shadow_type type;
 } shadow_names[] =
 {
@@ -2317,7 +2326,7 @@ make_windows_if_needed (XlwMenuWidget mw, int n)
   int start_at;
   XSetWindowAttributes xswa;
   Widget p;
-  int mask;
+  unsigned long mask;
   int depth;
   Visual *visual;
   window_state *windows;
@@ -2485,7 +2494,7 @@ remap_menubar (XlwMenuWidget mw)
 
   mw->menu.old_depth = new_depth;
 
-  /* refresh the last seletion */
+  /* refresh the last selection */
   selection_position.x = 0;
   selection_position.y = 0;
   display_menu (mw, last_same, new_selection == old_selection,
@@ -2996,10 +3005,6 @@ XlwMenuInitialize (Widget request, Widget new, ArgList args,
 {
   /* Get the GCs and the widget size */
   XlwMenuWidget mw = (XlwMenuWidget)new;
-
-  XSetWindowAttributes xswa;
-  int mask;
-
   Window window = RootWindowOfScreen (DefaultScreenOfDisplay (XtDisplay (mw)));
   Display *display = XtDisplay (mw);
 
@@ -3036,10 +3041,6 @@ XlwMenuInitialize (Widget request, Widget new, ArgList args,
   make_shadow_gcs      (mw);
   extract_font_extents (mw);
 
-  xswa.background_pixel = mw->core.background_pixel;
-  xswa.border_pixel     = mw->core.border_pixel;
-  mask = CWBackPixel | CWBorderPixel;
-
   mw->menu.popped_up              = False;
   mw->menu.pointer_grabbed        = False;
   mw->menu.next_release_must_exit = False;
@@ -3077,7 +3078,7 @@ XlwMenuRealize (Widget w, Mask *valueMask, XSetWindowAttributes *attributes)
 {
   XlwMenuWidget mw = (XlwMenuWidget)w;
   XSetWindowAttributes xswa;
-  int mask;
+  unsigned long mask;
 
   (*xlwMenuWidgetClass->core_class.superclass->core_class.realize)
     (w, valueMask, attributes);
@@ -3411,8 +3412,8 @@ xlw_map_menu (Time t)
   if (!mw->menu.pointer_grabbed)
     {
       XWindowAttributes ret;
-      Window parent,root;
-      Window *waste;
+      Window parent,root = 0UL;
+      Window *waste = NULL;
       unsigned int num_waste;
 
       lw_menu_active = True;
