@@ -2,8 +2,8 @@
 emacs_is_beta=
 emacs_major_version=21
 emacs_minor_version=4
-emacs_beta_version=15
-xemacs_codename="Security Through Obscurity"
+emacs_beta_version=20
+xemacs_codename="Double Solitaire"
 emacs_kit_version=
 infodock_major_version=4
 infodock_minor_version=0
